"use strict";

var logger = require('./logger');

/**
 * This static global object contains set of functions that returns/set data in
 * the Gui (html).
 */
var GuiConnector = {};

/**
 * Flag informing if the context menu is visible or not.
 */
GuiConnector.contextMenuVisible = false;

/**
 * Flag informing if selection menu is visible or not. Selection menu is
 * available when selcting polygon on the map and right clicking on it.
 */
GuiConnector.selectionMenuVisible = false;

/**
 * X coordinate of the mouse in a browser.
 */
GuiConnector.xPos = 0;

/**
 * Y coordinate of the mouse in a browser.
 */
GuiConnector.yPos = 0;

/**
 * List of GET params passed via url.
 */
GuiConnector.getParams = [];

GuiConnector.setCustomMap = function(customMap) {
  if (customMap === undefined) {
    throw new Error("CustomMap cannot be undefiend");
  }
  this._customMap = customMap;
};

GuiConnector.getCustomMap = function() {
  return this._customMap;
};

GuiConnector.init = function() {
  // bootstrap tab initialization
  $("ul.nav-tabs a").click(function(e) {
    e.preventDefault();
    $(this).tab('show');
  });

  // find GuiConnector.getParams
  document.location.search.replace(/\??(?:([^=]+)=([^&]*)&?)/g, function() {
    function decode(s) {
      return decodeURIComponent(s.split("+").join(" "));
    }
    GuiConnector.getParams[decode(arguments[1])] = decode(arguments[2]);
  });

  // forser browser to update mouse coordinates whenever mouse move
  jQuery(document).ready(function() {
    $(document).mousemove(function(e) {
      GuiConnector.updateMouseCoordinates(e.pageX, e.pageY);
    });
  });
};

/**
 * Returns name of the file with LCSB logo.
 * 
 * @param bigLogo
 *          {@link Boolean} value determining if we want to have big logo or
 *          small one
 */
GuiConnector.getLcsbLogoImg = function(bigLogo) {
  if (bigLogo) {
    return 'lcsb_logo_mid.png';
  } else {
    return 'lcsb_logo.png';
  }
};

/**
 * Returns name of the file with image that should be presented when we are
 * wainting for data to be loaded.
 */
GuiConnector.getLoadingImg = function() {
  return "icons/ajax-loader.gif";
};

/**
 * Returns home directory for images in the application.
 */
GuiConnector.getImgPrefix = function() {
  return "resources/images/";
};

/**
 * Hides right click menu.
 */
GuiConnector.hideRightClickMenu = function() {
  $(PrimeFaces.escapeClientId(ServerConnector.formIdentifier + ':contextMenu')).hide();
  this.contextMenuVisible = false;
};

/**
 * Returns <code>true</code> if right click menu is visible,
 * <code>false</code> otherwise.
 */
GuiConnector.isRightMenuVisible = function() {
  return this.contextMenuVisible;
};

/**
 * Shows right click menu.
 */
GuiConnector.showRightClickMenu = function(x, y) {
  $(PrimeFaces.escapeClientId(ServerConnector.formIdentifier + ':contextMenu')).css({
    top : y + 'px',
    left : x + 'px'
  }).show();
  this.contextMenuVisible = true;

  if (this.isSelectionMenuVisible) {
    this.hideSelectionMenu();
  }
};

/**
 * Hides selection menu.
 * 
 * @see selectionMenuVisible
 */
GuiConnector.hideSelectionMenu = function() {
  $(PrimeFaces.escapeClientId(ServerConnector.formIdentifier + ':selectionContextMenu')).hide();
  this.selectionMenuVisible = false;
};

/**
 * Returns <code>true</code> when selection menu is visible,
 * <code>false</code> otherwise.
 * 
 * @see selectionMenuVisible
 */
GuiConnector.isSelectionMenuVisible = function() {
  return this.selectionMenuVisible;
};

/**
 * Shows selection menu.
 * 
 * @see selectionMenuVisible
 */
GuiConnector.showSelectionMenu = function(x, y) {
  $(PrimeFaces.escapeClientId(ServerConnector.formIdentifier + ':selectionContextMenu')).css({
    top : y + 'px',
    left : x + 'px'
  }).show();
  this.selectionMenuVisible = true;

  if (this.isRightMenuVisible()) {
    this.hideRightClickMenu();
  }
};

/**
 * Updates coordinates of the mouse in the browser.
 */
GuiConnector.updateMouseCoordinates = function(x, y) {
  this.xPos = x;
  this.yPos = y;
};

/**
 * Return html tag for submap visualization.
 * 
 * @param id
 *          identifier of the submodel
 */
GuiConnector.getHtmlTagForSubmodelId = function(id) {
  return document.getElementById('_gmapForm:submodelDialog' + id);
};

/**
 * Returns js Primefaces object for submap visualization.
 * 
 * @param id
 *          identifier of the submodel
 */
GuiConnector.getJsPopupForSubmodelId = function(id) {
  return window['submodelDialog' + id];
};

/**
 * Opens popup for submap visualization.
 * 
 * @param id
 *          identifier of the submodel
 */
GuiConnector.openDialog = function(id) {
  var jsVar = GuiConnector.getJsPopupForSubmodelId(id);
  if (jsVar !== null) {
    var htmlTag = GuiConnector.getHtmlTagForSubmodelId(id);
    this.getCustomMap().openSubmodel(id, htmlTag, jsVar);
  }
  return false;
};

GuiConnector.referenceToHtml = function(reference) {
  var result;
  if (reference.summary !== undefined && reference.summary !== "") {
    result = '<div title="' + reference.summary + '">';
    result += '<a href="' + reference.link + '" target="_blank">' + reference.name + "</a>";
    // + reference.name + "(" + reference.type + ")</a>";
    result += "</div>";
    return result;
  } else {
    result = '<div><a href="' + reference.link + '" target="_blank">' + reference.name + "</a></div>";
    // + reference.name + "(" + reference.type + ")</a></div>";
    return result;
  }
};

GuiConnector.getObjectByPrimefaceId = function(id) {
  return $(PrimeFaces.escapeClientId(id));
};

GuiConnector.alert = function(message) {
  logger.error(message);
  alert(message);
};

module.exports = GuiConnector;
